{
  D3DX8Z - D3DX 8 unit.
  Copyright  2001 Lord Trancos.
  <lordtrancos@softhome.net>
}

unit D3DX8Z;

// --------------------------------------------------------------------

interface

uses Windows, Direct3D8, DX8GFX, D3DX8;

var
  D3DXSPR8 : ID3DXSprite = NIL;

// Main functions
function  InitD3DX: boolean;
procedure CloseD3DX;

// Texture/Surface functions
function D3DXLoadTextureFromFile(_filename: pChar; _cKey: TD3DColor): IDirect3DTexture8;
function  D3DXLoadSurfaceFromFile(_filename: pChar;
                                  _format: TD3DFormat): IDirect3DSurface8;

// Sprite functions
procedure D3DXDrawSpr(_texture: IDirect3DTexture8; _srcRect: PRect;
                      _translation, _scaling, _rotAxis: PD3DXVector2;
                      _rotation: single; _rgb: longint; _alfa: byte);

// --------------------------------------------------------------------

implementation

// --------------------------------------------------------------------
// Name : InitD3DX.
// Desc : Initializes D3DX Interfaces.
// --------------------------------------------------------------------

function  InitD3DX: boolean;

begin
  Result := false;

  // Get a ID3DXSprite interface.
  if failed(D3DXCreateSprite(D3DDEV8, D3DXSPR8)) then exit;

  // All right.
  Result := true;
end;

// --------------------------------------------------------------------
// Name : CloseD3DX.
// Desc : Closes D3DX Interfaces.
// --------------------------------------------------------------------

procedure CloseD3DX;

begin
  if assigned(D3DXSPR8) then D3DXSPR8 := NIL;
end;

// --------------------------------------------------------------------
// Name : D3DXLoadTextureFromFile.
// Desc : Loads a texture from a file (bmp, jpeg, png...)
//        _cKey -> ColorKey (ARGB format, 0 = none, $FF000000 = black,
//                 $FFFF0000 = red, $FF00FF00 = green, etc.)
//        _format must be 0 to use the default image format.
// --------------------------------------------------------------------

function D3DXLoadTextureFromFile(_filename: pChar; _cKey: TD3DColor): IDirect3DTexture8;
var _format: TD3DFormat;
begin
  Result := NIL;
  if failed(D3DXCreateTextureFromFileExA(D3DDEV8, _filename, 0, 0, 0, 0,
                                      _format, D3DPOOL_DEFAULT,
                                      D3DX_FILTER_NONE, D3DX_FILTER_NONE,
                                      _cKey, NIL, NIL, Result))
    then Result := NIL;
end;

// --------------------------------------------------------------------
// Name : D3DXLoadTextureFromFile.
// Desc : Loads a surface from a file (bmp, jpeg, png...)
//        _format must be 0 to use the default image format.
// --------------------------------------------------------------------

function D3DXLoadSurfaceFromFile(_filename: pChar;
                                 _format: TD3DFormat): IDirect3DSurface8;

var _r : TRect;
    _imginfo : TD3DXImageInfo;

begin
  Result := NIL;

  // Create a 16 bit dummy surface.
  if (failed(D3DDEV8.CreateImageSurface(4, 4, D3DFMT_R5G6B5,
      Result))) or (Result = NIL) then exit;

  // Load file into dummy surface.
  _r.Left := 0;
  _r.Top := 0;
  _r.Right := 3;
  _r.Bottom := 3;
  if (failed(D3DXLoadSurfaceFromFileA(Result, NIL, @_r, _filename, NIL,
                           D3DX_FILTER_NONE, 0, @_imginfo)))
    then Result := NIL;

  // All right ?
  if Result <> NIL then
    begin
      // Destroy dummy surface and create a new one with the image
      // dimensions.
      Result := NIL;

      // Create the surface.
{      if _format = 0 then} _format := _imginfo.Format;
      with _imginfo do
        if (failed(D3DDEV8.CreateImageSurface(Width, Height, _format,
            Result))) or (Result = NIL) then exit;

      // Load file into the surface.
      if (failed(D3DXLoadSurfaceFromFileA(Result, NIL, NIL, _filename,
                                NIL, D3DX_FILTER_NONE, 0, NIL)))
        then Result := NIL;
    end;
end;

// --------------------------------------------------------------------
// Name : D3DXDrawSpr.
// Desc : Draws a "2D" sprite.
// --------------------------------------------------------------------

procedure D3DXDrawSpr(_texture: IDirect3DTexture8; _srcRect: PRect;
                      _translation, _scaling, _rotAxis: PD3DXVector2;
                      _rotation: single; _rgb: longint; _alfa: byte);

begin
  D3DXSPR8.Draw(_texture, _srcRect, _scaling, _rotAxis, _rotation,
                _translation, (_alfa shl 24) + _rgb);
end;

// --------------------------------------------------------------------

end.
