unit PolyMap;
 {
       Polygon Map unit
        for SOLDAT

 Copyright  2002 Michal Marcinkowski
 }

interface
uses Direct3D8, Windows, DX8GFX, D3DX8, Graphics, dialogs, waypoints;

const
 MaxPolys = 200;
 MaxPolys2 = 10000;
 MIN_SECTOR = -15;
 MAX_SECTOR = 15;

 NMIN_SECTOR = -25;
 NMAX_SECTOR = 25;

type
  // The structure of the custom vertex.
  PCustomVertex = ^TCustomVertex;
  TCustomVertex = record
                    X, Y, Z, RHW : single; // Position for the vertex.
                    DColor       : TD3DCOLOR;  // Diffuse color.
                    tu,tv:Single;
                  end;

  TPolygon = object
   Vertices : array[1..3] of TCustomVertex;
  end;

  PCV = ^TCV;
  TCV = array[1..MaxPolys2,1..3] of TCustomVertex;

   TLoadSector = record
    Polys : array[1..200] of smallint;
    PolyCount : byte;
   end;

   TSaveSector = record
    Polys : array[1..200] of smallint;
    PolyCount : byte;
   end;

   TSector = record
    Polys : array[1..MaxPolys2] of smallint;
    PolyCount : byte;
   end;


  // The Polygon Map Structure
  TPolyMap = object
   VertexBuffer : IDirect3DVertexBuffer8;
   Polys : array[1..MaxPolys2] of TPolygon;
   Perp : array[1..MaxPolys2,1..3] of TD3DXVector3;
   PolyCount : integer;
   PosX, PosY  :integer;
   Texture : IDIRECT3DTEXTURE8;
   Sectors : array[NMIN_SECTOR..NMAX_SECTOR,NMIN_SECTOR..NMAX_SECTOR] of TSector;
   SectorsDivision : integer;
   BackgroundColor, BackgroundColor2 : longint;
   BackgroundVertexBuffer : IDirect3DVertexBuffer8;
   BackPoly : array[1..2] of TPolygon;
   Path : TNewWayPoints;
   PolyType : array[1..MaxPolys2] of byte;
  public
   function  Initialize:boolean;
   function  UpdateVertexBuffer:boolean;
   procedure Destroy;
   procedure Scroll(CX, CY: single);
   procedure Scale(SX, SY: single; CX, CY: single);
   procedure LoadTexture(filename : string);
   function PointInPoly(X,Y : single; i : integer):boolean;
   procedure BackgroundRender;
   function ClosestPerpendicular(j:integer; Pos : TD3DXVector2; var D : single):integer;
  end;


  const
   // Flexible vector format (FVF) description.
   // Describes the custom vertex structure (TCustomVertex).
   D3DFVF_CUSTOMVERTEX = D3DFVF_XYZRHW or D3DFVF_DIFFUSE or D3DFVF_TEX1;


  function GetCustomVertex(_x, _y, _z, _rhw, _u, _v: single;
                         _dcolor: dWord; PosX, PosY:integer): TCustomVertex;

 var
  SECTOR_NUM : integer = 25;

implementation
uses Calc, Unit1;

function GetCustomVertex(_x, _y, _z, _rhw, _u, _v: single;
                         _dcolor: dWord; PosX, PosY:integer): TCustomVertex;
begin
  with Result do
    begin
      X := _x;
      Y := _y;
      Z := _z;
      RHW := _rhw;
      if (_u=0.0)and(_v=0.0) then
      begin
       TU:= ((_x-320)/640)*4;
       TV:= ((_y-240)/480)*4
      end else
      begin
       TU:= _u;
       TV:= _v;
      end; 
      DColor := _dcolor;
    end;
end;

////////||| TPOLYMAP.INITIALIZE |||\\\\\\\
function TPolyMap.Initialize:boolean;
var _ptr : pByte;
begin
  Result:= false;

  PolyCount:= 0;
  PosX:= 0; PosY:= 0;

  // Create a vertex buffer.
  if failed(D3DDEV8.CreateVertexBuffer(SizeOf(Polys), 0,
                                       D3DFVF_CUSTOMVERTEX,
                                       D3DPOOL_DEFAULT,
                                       VertexBuffer)) then exit;

  // Lock, fill and unlock vertex buffer.
  if failed(VertexBuffer.Lock(0, SizeOf(Polys), _ptr, 0)) then exit;
  Move(Polys, _ptr^, SizeOf(Polys));
  VertexBuffer.Unlock;

    //Background -  Create a vertex buffer.
    if failed(D3DDEV8.CreateVertexBuffer(SizeOf(BackPoly), 0,
                                         D3DFVF_CUSTOMVERTEX,
                                         D3DPOOL_DEFAULT,
                                         BackgroundVertexBuffer)) then exit;

    //Background -  Lock, fill and unlock vertex buffer.
    if failed(VertexBuffer.Lock(0, SizeOf(BackPoly), _ptr, 0)) then exit;
    Move(BackPoly, _ptr^, SizeOf(BackPoly));
    BackgroundVertexBuffer.Unlock;
                                        

  Result:= true;
end;

////////||| TPOLYMAP.DESTROY |||\\\\\\\\\
procedure TPolyMap.Destroy;
begin
 VertexBuffer:= NIL;
 Texture:= NIL;
end;

////////||| TPOLYMAP.UPDATEVERTEXBUFFER |||\\\\\\\\\
function TPolyMap.UpdateVertexBuffer:boolean;
var    _ptr : pByte;
begin
  Result:= false;

  // Lock, fill and unlock vertex buffer.
  if failed(VertexBuffer.Lock(0, SizeOf(Polys), _ptr, 0)) then exit;
  Move(Polys, _ptr^, SizeOf(Polys));
  VertexBuffer.Unlock;

  if failed(BackgroundVertexBuffer.Lock(0, SizeOf(BackPoly), _ptr, 0)) then exit;
  Move(BackPoly, _ptr^, SizeOf(BackPoly));
  BackgroundVertexBuffer.Unlock;

  Result:= true;
end;

////////||| TPOLYMAP.SCROLL |||\\\\\\\\\
procedure TPolyMap.Scroll(CX, CY: single);
var
 i, j : integer;
 _ptr : PCV;
begin
 if failed(VertexBuffer.Lock(0, SizeOf(Polys), pByte(_ptr), 0)) then exit;

 for i:= 1 to PolyCount do
 begin
  with Polys[i] do
  for j:= 1 to 3 do
  begin
   _ptr[i][j].X:= 320+Vertices[j].X-CX;
   _ptr[i][j].Y:= 240+Vertices[j].Y-CY;
  end; {with Polys / for j}
 end; {for i}

  VertexBuffer.Unlock;


 if failed(BackgroundVertexBuffer.Lock(0, SizeOf(BackPoly), pByte(_ptr), 0)) then exit;

 for i:= 1 to 2 do
 begin
  with BackPoly[i] do
  for j:= 1 to 3 do
  begin
   _ptr[i][j].X:= 320+Vertices[j].X-CX;
   _ptr[i][j].Y:= 240+Vertices[j].Y-CY;
  end; {with Polys / for j}
 end; {for i}

  BackgroundVertexBuffer.Unlock;
end;

////////||| TPOLYMAP.SCALE |||\\\\\\\\\
procedure TPolyMap.Scale(SX, SY: single; CX, CY: single);
var
 i, j : integer;
 _ptr : PCV;
begin
 if failed(VertexBuffer.Lock(0, SizeOf(Polys), pByte(_ptr), 0)) then exit;

 for i:= 1 to PolyCount do
 begin
  with Polys[i] do
  for j:= 1 to 3 do
  begin
   _ptr[i][j].X:= SX*Vertices[j].X-CX;
   _ptr[i][j].Y:= SY*Vertices[j].Y-CY;
  end; {with Polys / for j}
 end; {for i}

  VertexBuffer.Unlock;


 if failed(BackgroundVertexBuffer.Lock(0, SizeOf(BackPoly), pByte(_ptr), 0)) then exit;

 for i:= 1 to 2 do
 begin
  with BackPoly[i] do
  for j:= 1 to 3 do
  begin
   _ptr[i][j].X:= SX*Vertices[j].X-CX;
   _ptr[i][j].Y:= SY*Vertices[j].Y-CY;
  end; {with Polys / for j}
 end; {for i}

  BackgroundVertexBuffer.Unlock;
end;


////////||| TPOLYMAP.LOADTEXTURE |||\\\\\\\\\
procedure TPolyMap.LoadTexture(filename : string);
begin
 // Use D3DX to create a texture from a file based image
 if( FAILED( D3DXCreateTextureFromFile( D3DDEV8, PChar(filename),
                                      Texture ) ) ) then
   begin
    MessageBox(0, pchar('Cannot load texture '+filename), 'Error', MB_ICONERROR);
    exit;
   end;
end;


//*************##############***************//

//////////////////// POINTINPOLY //////////////////
function TPolyMap.PointInPoly(X,Y : single; i : integer):boolean;
var
 U : TD3DXVector3;
 D : single;
begin
 Result:= false;

  U.X:= X-Polys[i].Vertices[1].X; U.Y:= Y-Polys[i].Vertices[1].Y;
  D:= Perp[i][1].X * U.X + Perp[i][1].Y * U.Y;
  if D/abs(D)=-1 then exit;

  U.X:= X-Polys[i].Vertices[2].X; U.Y:= Y-Polys[i].Vertices[2].Y;
  D:= Perp[i][2].X * U.X + Perp[i][2].Y * U.Y;
  if D/abs(D)=-1 then exit;

  U.X:= X-Polys[i].Vertices[3].X; U.Y:= Y-Polys[i].Vertices[3].Y;
  D:= Perp[i][3].X * U.X + Perp[i][3].Y * U.Y;
  if D/abs(D)=-1 then exit;

 Result:= true;
end;


procedure TPolyMap.BackgroundRender;
begin
  // Set the texture that the will be used to render.
  D3DDEV8.SetTexture(0, NIL);
  // Set the stream source.
  D3DDEV8.SetStreamSource( 0, BackgroundVertexBuffer, sizeof(TCustomVertex));
  // Set the vertex shader.
  D3DDEV8.SetVertexShader(D3DFVF_CUSTOMVERTEX);
  // Render the vertices.
  D3DDEV8.DrawPrimitive(D3DPT_TRIANGLELIST, 0, 2);
end;



 function TPolyMap.ClosestPerpendicular(j:integer; Pos : TD3DXVector2; var D : single):integer;
 var
  PX, PY : array[1..3] of single;
  P1, P2 : TD3DXVector2;
  D1, D2, D3 : single;
  EdgeV1, EdgeV2 : integer;
 begin
         PX[1]:= Polys[j].Vertices[1].X; PY[1]:= Polys[j].Vertices[1].Y;
         PX[2]:= Polys[j].Vertices[2].X; PY[2]:= Polys[j].Vertices[2].Y;
         PX[3]:= Polys[j].Vertices[3].X; PY[3]:= Polys[j].Vertices[3].Y;

    {find closest edge}
    P1.X:= PX[1]; P1.Y:= PY[1];
    P2.X:= PX[2]; P2.Y:= PY[2];
     D1:= PointLineDistance(P1,P2,Pos); D:= D1;
     EdgeV1:= 1; EdgeV2:= 2;
    P1.X:= PX[2]; P1.Y:= PY[2];
    P2.X:= PX[3]; P2.Y:= PY[3];
     D2:= PointLineDistance(P1,P2,Pos);
     if D2<D1 then
     begin
      EdgeV1:= 2; EdgeV2:= 3;
      D:= D2;
     end;
    P1.X:= PX[3]; P1.Y:= PY[3];
    P2.X:= PX[1]; P2.Y:= PY[1];
     D3:= PointLineDistance(P1,P2,Pos);
     if (D3<D2)and(D3<D1) then
     begin
      EdgeV1:= 3; EdgeV2:= 1;
      D:= D3;
     end;

    if (EdgeV1=1)and(EdgeV2=2) then
     Result:= 1;
    if (EdgeV1=2)and(EdgeV2=3) then
     Result:= 2;
    if (EdgeV1=3)and(EdgeV2=1) then
     Result:= 3;
 end;
//*************##############***************//


end.


